-- -------------------------------------------------------------------------- --
-- PickPocketTimer by kunda                                                   --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- PickPocketTimer shows a small movable frame when you target a unit whose   --
-- pocket was picked by you. This frame shows a countdown which indicates     --
-- when this units' pocket can picked again.                                  --
--                                                                            --
-- This addon was mainly created as a little helper addon for the achievement --
-- 'Insane in the Membrane'.                                                  --
--                                                                            --
-- Countdown Textcolor means:                                                 --
-- red    = Pick Pocket NOT possible    = 480-180 seconds (8:00-3:00 minutes) --
-- yellow = Pick Pocket MAY BE possible = 179-  0 seconds (2:59-0:00 minutes) --
--                                                                            --
-- This Pocket-'Respawn Times' are based on empiric experience. The result of --
-- my tests varied from 314 (5:14) to 472 (7:52) seconds.                     --
-- Conclusion:                                                                --
--  After 300 seconds there is a chance to pick the same units' pocket again. --
--  After 480 seconds every units' pocket is 'respawned'.                     --
--  If you have other results, please let me know, thanks.                    --
--                                                                            --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- NOTE:                                                                      --
--  Unit data (GUID and time) is saved in a temp table not in SavedVariable.  --
--  UIReload/Disconnect/Logout/Exit will reset this temp table!               --
--                                                                            --
--  WORKAROUND FOR AN ANNOYING BUG:                                           --
--   If auto loot is enabled and an units' pocket was picked while moving(!)  --
--   the event LOOT_SLOT_CLEARED is triggered only once, regardless how many  --
--   items are in the loot! This does not happen if you don't move!           --
--                                                                            --
--   To save such unit data to TimerDB, we need to check if enough bag slots  --
--   are available (only normal bags and no stackable check).                 --
--                                                                            --
--   This means:                                                              --
--   If you have auto loot enabled                                            --
--   and move while picking a pocket                                          --
--   and there are not enough free bag slots to loot all                      --
--   and all loot items stack with an item in a bag                           --
--   -> NO unit data is saved to TimerDB.                                     --
--                                                                            --
-- Hint:                                                                      --
--  If you have enough free normal bag slots every 'Pick Pocket' is correctly --
--  saved to the temp table (TimerDB).                                        --
--                                                                            --
-- -------------------------------------------------------------------------- --

--  ------------------------------------------------------------------------  --
--                                    STOP                                    --
--                   You must be a Rogue to enter this code.                  --
--                    If you are not a Rogue please leave.                    --
          if select(2, UnitClass("player")) ~= "ROGUE" then return end          
--  ------------------------------------------------------------------------  --

-- ---------------------------------------------------------------------------------------------------------------------
PickPocketTimer = CreateFrame("Frame", "PickPocketTimer_Frame")
PickPocketTimer_Options = {}      -- SavedVariable options table

local CountdownStatusMAYBE = 179  -- 179 seconds (2:59)
local CountdownStatusEND   = 480  -- 480 seconds (8:00)

local TimerDB = {}                -- table with Pick Pocket Duration
local PickPocketIsActive = false  -- Pick Pocket check value (necessary for LOOT_* events)
local PickPocketGUID              -- Pick Pocket dstGUID (from COMBAT_LOG_EVENT_UNFILTERED)
local NumPickPocketLoot = 0       -- number of items in pocket loot (LootFrame.numLootItems)
local FreeBagSlots                -- availabe bag slots (only normal bags - GetItemFamily(bagLink) = 0)
local TargetGUID                  -- current target GUID
local PlayerGUID                  -- your GUID (set after event PLAYER_ENTERING_WORLD triggered)
local InCombat = false            -- false = out of combat | true = in combat

local _G = _G
local floor = _G.floor
local UnitGUID = _G.UnitGUID
local UnitIsDead = _G.UnitIsDead
local GetTime = _G.GetTime
local ContainerIDToInventoryID = _G.ContainerIDToInventoryID
local GetInventoryItemLink = _G.GetInventoryItemLink
local GetContainerNumFreeSlots = _G.GetContainerNumFreeSlots
local GetItemFamily = _G.GetItemFamily
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function DigitalTime(time)
	local time = CountdownStatusEND - time
	local minutes = floor(time/60)
	local seconds = time-(minutes*60)

	if seconds < 10 then
		seconds = "0"..seconds
	end
	
	local color = "|cffff0000" -- red    = Pick Pocket NOT possible
	if time <= CountdownStatusMAYBE then
		color = "|cffffff00"     -- yellow = Pick Pocket MAY BE possible
	end
	
	return color..minutes..":"..seconds.."|r"
end

local function DurationCheck(currentTime)
	for k,v in pairs(TimerDB) do
		if (currentTime - v) >= CountdownStatusEND then
			TimerDB[k] = nil
		end
	end
end

local function UpdateFrame()
	if InCombat then
		PickPocketTimer:Hide()
	else
		if TargetGUID and TargetGUID == UnitGUID("target") and TimerDB[TargetGUID] then
			local currentTime = floor(GetTime())
			if UnitIsDead("target") then
				TimerDB[TargetGUID] = nil
				DurationCheck(currentTime)
				PickPocketTimer:Hide()
				return
			end
			local leftTime = currentTime - TimerDB[TargetGUID]
			if leftTime >= CountdownStatusEND then
				DurationCheck(currentTime)
				PickPocketTimer:Hide()
			else
				PickPocketTimer_FrameText:SetText(DigitalTime(leftTime))
			end
		end
	end
end

local function CheckTarget(currentTime)
	if InCombat then return end
	DurationCheck(currentTime)
	if TargetGUID and TimerDB[TargetGUID] then
		PickPocketTimer:Show()
		PickPocketTimer_FrameText:SetText(DigitalTime(currentTime - TimerDB[TargetGUID]))
		PickPocketTimer:SetScript("OnUpdate", UpdateFrame)
	else
		PickPocketTimer:Hide()
		PickPocketTimer:SetScript("OnUpdate", nil)
	end
end

local function IsBag(bag)
	local invID = ContainerIDToInventoryID(bag)
	local bagLink = GetInventoryItemLink("player", invID)
	if bagLink then
		return bagLink
	else
		return false
	end
end

local function GetFreeBagSlots()
	local freeSlots = GetContainerNumFreeSlots(0)
	local containerBagType = 0
	for bag = 1, NUM_BAG_SLOTS do
		local bagLink = IsBag(bag)
		if bagLink then
			containerBagType = GetItemFamily(bagLink)
			if containerBagType == 0 then
				freeSlots = freeSlots + GetContainerNumFreeSlots(bag)
			end
		end
	end
	return freeSlots
end

local function SaveUnitToTimerDB()
	local currentTime = floor(GetTime())
	TimerDB[PickPocketGUID] = currentTime -- print("|cffffff7fSAVED:|r "..PickPocketGUID.." - "..currentTime)
	CheckTarget(currentTime)
	NumPickPocketLoot = 0 -- necessary because empty loot does not trigger LOOT_OPENED if auto loot is enabled
end

local function SaveFramePos()
	PickPocketTimer_Options.posX = PickPocketTimer_Frame:GetLeft()
	PickPocketTimer_Options.posY = PickPocketTimer_Frame:GetTop()
end

local function Init()
	PickPocketTimer:SetClampedToScreen(true)
	PickPocketTimer:SetMovable(true)
	PickPocketTimer:EnableMouse(true)
	PickPocketTimer:SetPoint("CENTER", UIParent, 0, 200)
	PickPocketTimer:SetFrameStrata("DIALOG")
	PickPocketTimer:SetWidth(50)
	PickPocketTimer:SetHeight(20)
	PickPocketTimer:Hide()
	PickPocketTimer:SetScript("OnMouseDown", function() PickPocketTimer_Frame:StartMoving() end)
	PickPocketTimer:SetScript("OnMouseUp", function() PickPocketTimer_Frame:StopMovingOrSizing() SaveFramePos() end)
	PickPocketTimer:SetScript("OnDragStop", function() PickPocketTimer_Frame:StopMovingOrSizing() end)

	local button = CreateFrame("Button", "PickPocketTimer_FrameButton", PickPocketTimer_Frame)
	button:SetFrameLevel(PickPocketTimer_Frame:GetFrameLevel()+1)
	button:SetWidth(20)
	button:SetHeight(20)
	button:ClearAllPoints()
	button:SetPoint("LEFT", "PickPocketTimer_Frame", "LEFT", -2, 0)
	button:SetScript("OnMouseDown", function() PickPocketTimer_Frame:StartMoving() end)
	button:SetScript("OnMouseUp", function() PickPocketTimer_Frame:StopMovingOrSizing() SaveFramePos() end)
	button:SetScript("OnDragStop", function() PickPocketTimer_Frame:StopMovingOrSizing() end)

	local normalTexture = button:CreateTexture(nil, "BACKGROUND")
	normalTexture:SetAllPoints(button)
	normalTexture:SetTexture("Interface\\Icons\\INV_Misc_Bag_11")
	button:SetNormalTexture(normalTexture)

	local text = PickPocketTimer:CreateFontString("PickPocketTimer_FrameText", "OVERLAY", "GameFontHighlightSmall")
	text:SetPoint("LEFT", PickPocketTimer_FrameButton, "RIGHT", 4, 0)
	
	local texture = PickPocketTimer:CreateTexture(nil, "BACKGROUND")
	texture:SetAllPoints(PickPocketTimer)
	texture:SetTexture(0, 0, 0, 1)
	
	if PickPocketTimer_Options.IssueMessage == nil then PickPocketTimer_Options.IssueMessage = false end

	if PickPocketTimer_Options.posX then
		PickPocketTimer_Frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", PickPocketTimer_Options.posX, PickPocketTimer_Options.posY)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function OnEvent(self, event, ...)
	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		--    1, 2        , 3, 4         , 5, 6, 7       , 8, 9, 10    
		local _, eventName, _, sourceGUID, _, _, destGUID, _, _, spellId = ...
		if sourceGUID ~= PlayerGUID then return end
		if not destGUID then return end
		if spellId ~= 921 then return end
		if eventName == "SPELL_CAST_SUCCESS" then
			PickPocketIsActive = true
			PickPocketGUID = destGUID
		end
	elseif event == "PLAYER_TARGET_CHANGED" then
		TargetGUID = UnitGUID("target")
		CheckTarget(floor(GetTime()))
	elseif event == "PLAYER_REGEN_DISABLED" then
		InCombat = true
	elseif event == "PLAYER_REGEN_ENABLED" then
		InCombat = false
		TargetGUID = UnitGUID("target")
		if TargetGUID then
			CheckTarget(floor(GetTime()))
		end
	elseif event == "LOOT_OPENED" then
		if PickPocketIsActive then
			NumPickPocketLoot = LootFrame.numLootItems
			if GetCVar("AutoLootDefault") == "1" then
				local firstSlotIsCoin = 0
				if LootSlotIsCoin(1) then
					firstSlotIsCoin = 1
				end
				FreeBagSlots = false
				if GetFreeBagSlots() >= (NumPickPocketLoot - firstSlotIsCoin) then
					FreeBagSlots = true
				end
			end
		end
	elseif event == "LOOT_SLOT_CLEARED" then
		if PickPocketIsActive then
			NumPickPocketLoot = NumPickPocketLoot - 1
			if NumPickPocketLoot == 0 then
				PickPocketIsActive = false
				SaveUnitToTimerDB()
			end
		end
	elseif event == "LOOT_CLOSED" then
		if PickPocketIsActive then
			if NumPickPocketLoot == 0 then -- this is the case if target has no loot in the pocket (empty loot does not trigger LOOT_OPENED if auto loot is enabled)
				SaveUnitToTimerDB()
			else
				if GetCVar("AutoLootDefault") == "1" then
					if FreeBagSlots then
						if NumPickPocketLoot > 0 then
							SaveUnitToTimerDB()
						end
					else
						if not PickPocketTimer_Options.IssueMessage then
							print("|cffffff7fPickPocketTimer:|r Unit data not saved to TimerDB.")
							print("|cffffff7fPickPocketTimer:|r Please read the NOTE at top of PickPocketTimer.lua for more information.")
							print("|cffffff7fPickPocketTimer:|r This message is only displayed once.")
							PickPocketTimer_Options.IssueMessage = true
						end
					end
				end
			end
		end
		PickPocketIsActive = false
	elseif event == "PLAYER_ENTERING_WORLD" then
		PlayerGUID = UnitGUID("player")
		Init()
		PickPocketTimer:UnregisterEvent("PLAYER_ENTERING_WORLD")
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

PickPocketTimer:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
PickPocketTimer:RegisterEvent("PLAYER_TARGET_CHANGED")
PickPocketTimer:RegisterEvent("PLAYER_REGEN_DISABLED")
PickPocketTimer:RegisterEvent("PLAYER_REGEN_ENABLED")
PickPocketTimer:RegisterEvent("LOOT_OPENED")
PickPocketTimer:RegisterEvent("LOOT_SLOT_CLEARED")
PickPocketTimer:RegisterEvent("LOOT_CLOSED")
PickPocketTimer:RegisterEvent("PLAYER_ENTERING_WORLD")
PickPocketTimer:SetScript("OnEvent", OnEvent)